/*
 * Copyright 2010-2011 Heads Up Development Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.headsupdev.license.manager;

import org.headsupdev.license.License;
import org.headsupdev.license.LicenseEncoder;
import org.headsupdev.license.LicenseUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.security.Key;

/**
 * TODO: Document me
 * <p/>
 * Created: 15/03/2012
 *
 * @author Andrew Williams
 * @since 1.1
 */
public class AddLicenseForm
        extends JPanel {
    private JPanel form;
    private JButton createLicenseButton;
    private JTextPane licensedTo;

    public AddLicenseForm(final LicenseManager manager) {
        setLayout(new BorderLayout());
        add(form, BorderLayout.CENTER);
        createLicenseButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                String to = licensedTo.getText();
                String fileName = to + ".license";

                int i = 1;
                String uniqueFileName = fileName;
                while (new File(manager.getConfig().getLicenseDirectory(), uniqueFileName).exists()) {
                    uniqueFileName = fileName + i;
                    i++;
                }

                File licenseFile = new File(manager.getConfig().getLicenseDirectory(), uniqueFileName);
                License license = manager.createLicense();
                license.setLicensedTo(licensedTo.getText());

                String licenseContent = "";
                try {
                    Key prv = LicenseUtils.deserialiseKey(manager.getConfig().getPrivateKeyFile());
                    Key shr = LicenseUtils.deserialiseKey(manager.getConfig().getSharedKeyFile());

                    LicenseEncoder encoder = new LicenseEncoder();

                    encoder.setPrivateKey(prv);
                    encoder.setSharedKey(shr);

                    licenseContent = encoder.encodeLicense(license);
                    LicenseUtils.writeLicenseFile(licenseContent, licenseFile);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                licenseAdded(license, licenseFile);
            }
        });
    }

    public void licenseAdded(License license, File file) {
        // no-op
    }

    {
// GUI initializer generated by IntelliJ IDEA GUI Designer
// >>> IMPORTANT!! <<<
// DO NOT EDIT OR ADD ANY CODE HERE!
        $$$setupUI$$$();
    }

    /**
     * Method generated by IntelliJ IDEA GUI Designer
     * >>> IMPORTANT!! <<<
     * DO NOT edit this method OR call it in your code!
     *
     * @noinspection ALL
     */
    private void $$$setupUI$$$() {
        form = new JPanel();
        form.setLayout(new GridBagLayout());
        final JLabel label1 = new JLabel();
        label1.setText("License To");
        GridBagConstraints gbc;
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.weightx = 0.3;
        gbc.anchor = GridBagConstraints.WEST;
        form.add(label1, gbc);
        licensedTo = new JTextPane();
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 2;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        form.add(licensedTo, gbc);
        final JLabel label2 = new JLabel();
        label2.setFont(new Font(label2.getFont().getName(), Font.BOLD, 18));
        label2.setText("Details for new License:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 2;
        gbc.anchor = GridBagConstraints.WEST;
        form.add(label2, gbc);
        final JLabel label3 = new JLabel();
        label3.setText("Expiry date");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.anchor = GridBagConstraints.WEST;
        form.add(label3, gbc);
        final JLabel label4 = new JLabel();
        label4.setText(" or");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(0, 20, 0, 0);
        form.add(label4, gbc);
        final JLabel label5 = new JLabel();
        label5.setText("Valid days");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 5;
        gbc.anchor = GridBagConstraints.WEST;
        form.add(label5, gbc);
        final JPanel spacer1 = new JPanel();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 6;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.VERTICAL;
        form.add(spacer1, gbc);
        final JPanel spacer2 = new JPanel();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.fill = GridBagConstraints.VERTICAL;
        form.add(spacer2, gbc);
        createLicenseButton = new JButton();
        createLicenseButton.setText("Create License");
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 7;
        gbc.anchor = GridBagConstraints.EAST;
        form.add(createLicenseButton, gbc);
    }

    /**
     * @noinspection ALL
     */
    public JComponent $$$getRootComponent$$$() {
        return form;
    }
}
